/*
 * Dfmpr:  Derivative free minimization palomares rodriguez
 *
 * Copyright (c) 2002-2013
 *   Arno Formella (formella@ei.uvigo.es)
 *   Ubaldo Garcia-Palomares (garciap@usb.ve)
 *   Jose F. Rodriguez (jrodri@usb.ve)
 */

/*
 * Academic Free License v2.0
 * 
 * http://www.opensource.org/licenses/
 */

#ifndef __MCL_DFMPR_H__
#define __MCL_DFMPR_H__

#include "optimize.h"

#ifdef HAVE_MCL_RANDOM
#include "MCl_Random.h"
#else
#include <cstdlib>
#endif

namespace mcl {

template<typename V>
class Dfmpr : public Optimize<V> {
private:
#ifdef HAVE_MCL_RANDOM
  mcl::Random R;
#endif
  const double tau;     //!< initial size, should be larger than 0.001
  const double delta;   //!< lower limit
  /// constant used during expansion.
  /// must be strictly larger than 1.0
  const double expansion_constant;
  /// constant used during contraction.
  /// must be strictly smaller than 1.0
  const double contraction_constant;
  void CheckLimits(V* inf, V* sup) const;
  void GenerateD(
    const V& u,  //!< where to generate
    int index,
    V& d         //!< the generated directions
  ) const;
  double Median(
    double a,
    double b,
    double c
  ) const;
  void Median(
    const V& a,
    const V& b,
    const V& c,
    V& x
  ) const;
  /// copy-construction is not allowed (up to now ...)
  Dfmpr(const Dfmpr& O);
  /// direct assignment is not allowed (up to now ...)
  Dfmpr& operator=(const Dfmpr& O);
public:
  Dfmpr(
    OptimizationProblem<V>& problem,
    unsigned int max_iterations,
    unsigned int max_evaluations,
    double tolerance=0.00001,
    double tau=0.1,
    double delta_magic=0.01,
    double expansion_constant=1.8,
    double contraction_constant=0.2
  );
  virtual ~Dfmpr();
  double Delta(void) const { return delta; }
  double ContractionConstant(void) const { return contraction_constant; }
  double ExpansionConstant(void) const { return expansion_constant; }
  virtual typename Optimize<V>::Optresult Run(
    V& v,
    V* inf=0,
    V* sup=0
  ) const;
  double Rand(void) const;
  void SetRandomSeed(void) const;
  void SetSeed(unsigned int seed) const;
  double Tau(void) const { return tau; }
};

#include "dfmpr.cc"

}

#endif
